library zhihu_daily.models;

import 'dart:convert';

/// 启动界面图片
class StartImage {
  /// 供显示的图片版权信息
  String text;

  /// 图片的url
  String img;

  /// 从json数据创建StartImage
  StartImage.fromJson(Map json) {
    text = json['text'];
    img = json['img'];
  }

  String toString() {
    return '{text: $text, img: $img}';
  }
}

/// 消息
class News {
  /// 日期
  String date;

  /// 标题
  String title;

  /// 图片地址
  String image;

  // 未知类型
  // int type;

  /// 消息内容[Story]的id
  int storyId;

  /// 是否为头条
  bool isTop = false;

  /// 从json数据创建News
  News.fromJson(Map json, {bool top: false}) {
    if (json.containsKey('date')) {
      date = json['date'];
    }
    title = json['title'];

    if (json.containsKey('image')) {
      image = json['image'];
    } else if (json.containsKey('images')) {
      image = json['images'][0];
    }

    storyId = json['id'];
    isTop = top;
  }

  String toString() {
    return '{title: $title, image: $image}';
  }
}

/// 消息内容
class Story {
  /// HTML格式的内容
  String body;

  /// 图片的内容提供方。为了避免被起诉非法使用图片，在显示图片时最好附上其版权信息。
  String imageSource;

  /// 标题
  String title;

  /// 与[News]的图片分辨率不同，这里获得的是在文章浏览界面中使用的大图。
  String image;

  /// 供在线查看内容与分享至 SNS 用的 URL
  String shareUrl;

  /// 文章的推荐者
  List<Map<String, String>> recommenders;

  /// 所属栏目[Section]
  Section section;

  /// 所属类型
  int type;

  /// id
  int id;

  Story.fromJson(Map json) {
    body = HTML_ESCAPE.convert(json['body']);
    imageSource = json['image_source'];
    title = json['title'];
    image = json['image'];
    shareUrl = json['share_url'];
    recommenders = json['recommenders'];
    type = json['type'];
    id = json['id'];

    if (json.containsKey('section')) {
      var map = json['section'];
      section = new Section(
          thumbnail: map['thumbnail'], id: map['id'], name: map['name']);
    }
  }

  String toString() {
    return '{title: $title, image: $image}';
  }
}

/// 栏目
class Section {
  /// 缩略图
  String thumbnail;

  /// 栏目id
  int id;

  /// 栏目名称
  String name;

  Section({this.thumbnail, this.id, this.name});

  String toString() {
    return '{name: $name, thumbnail: $thumbnail}';
  }
}

/// 消息的附加信息
class StoryExtra {
  /// 长评论总数
  int longComments;

  /// 点赞总数
  int popularity;

  /// 短评论总数
  int shortComments;

  /// 评论总数
  int comments;

  /// 所属消息id
  int storyId;

  StoryExtra.fromJson(Map json) {
    longComments = json['long_comments'];
    popularity = json['popularity'];
    shortComments = json['short_comments'];
    comments = json['comments'];
  }
}

/// 评论
class Comment {
  /// 评论作者
  String author;

  /// 评论者id
  int id;

  /// 评论内容
  String content;

  /// 评论所获点赞数量
  int likes;

  /// 评论时间
  String time;

  /// 评论者的头像地址
  String avatar;

  /// 是否为长评论
  bool isLong = false;

  /// 是否为短评论
  bool isShort = false;

  Comment.fromJson(Map json, [bool short = true]) {
    author = json['author'];
    id = json['id'];
    content = json['content'];
    likes = json['likes'];
    time = json['time'];
    avatar = json['avatar'];

    if (short) {
      isShort = true;
    } else {
      isLong = true;
    }
  }

  String toString() {
    return '{author: $author, content: $content}';
  }
}

/// 主题
class Theme {
  /// 图片地址
  String thumbnail;

  /// 介绍
  String description;

  /// id
  int id;

  /// 名称
  String name;

  Theme.fromJson(Map json) {
    thumbnail = json['thumbnail'];
    description = json['description'];
    id = json['id'];
    name = json['name'];
  }

  String toString() {
    return '{thumbnail: $thumbnail, description: $description}';
  }
}

/// 主题内容
class ThemeContent {
  String description;
  String background;
  String name;
  String image;
  List<Map<String, String>> editors;
  String imageSource;
  List<News> newsList;

  ThemeContent.fromJson(Map json) {
    description = json['description'];
    background = json['background'];
    name = json['name'];
    image = json['image'];
    editors = json['editors'];
    imageSource = json['image_source'];

    var stories = json['stories'];
    newsList = <News>[];
    for (var item in stories) {
      newsList.add(new News.fromJson(item));
    }
  }

  String toString() {
    return '{name: $name, description: $description}';
  }
}
